% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Pre_processing_hashcodes.R
\name{Pre_processing_hashcodes}
\alias{Pre_processing_hashcodes}
\title{Pre-process taxa in a phyloseq or TSE object by merging ASVs/OTUs using hashcodes}
\usage{
Pre_processing_hashcodes(
  obj,
  hashcodes,
  merge_method = c("sum", "max"),
  output_file = NULL
)
}
\arguments{
\item{obj}{A \code{phyloseq} or \code{TreeSummarizedExperiment} object.}

\item{hashcodes}{A character vector of OTU hashcodes to be merged.}

\item{merge_method}{Merge strategy to apply to counts: \code{"sum"} (default) or \code{"max"}.}

\item{output_file}{Optional path to save the resulting object as an \code{.rds} file.}
}
\value{
A processed \code{phyloseq} or \code{TreeSummarizedExperiment} object with merged ASVs/OTUs.
}
\description{
Merges ASVs/OTUs using a list of exact hashcodes (typically OTU IDs from QIIME2).
Supports both \code{phyloseq} and \code{TreeSummarizedExperiment} (TSE) objects. Ensures that
phylogenetic trees and reference sequences are retained and pruned as needed to remain
consistent with merged taxa.
}
\examples{
if (requireNamespace("DspikeIn", quietly = TRUE)) {
  data("physeq_16SOTU", package = "DspikeIn")

  # Example 1: phyloseq input
  subset_physeq <- phyloseq::subset_taxa(
    physeq_16SOTU,
    Species \%in\% c("Tetragenococcus_halophilus", "Tetragenococcus_sp.")
  )
  hashcodes_phy <- rownames(phyloseq::otu_table(subset_physeq))

  merged_physeq <- Pre_processing_hashcodes(
    obj = physeq_16SOTU,
    hashcodes = hashcodes_phy,
    merge_method = "sum"
  )

  # Example 2: TreeSummarizedExperiment (TSE) input
  tse_16SOTU <- convert_phyloseq_to_tse(physeq_16SOTU)
  row_annots <- SummarizedExperiment::rowData(tse_16SOTU)
  subset_tse <- tse_16SOTU[
    which(row_annots$Species \%in\% c("Tetragenococcus_halophilus", "Tetragenococcus_sp.")),
  ]
  hashcodes_tse <- rownames(SummarizedExperiment::assay(subset_tse, "counts"))

  merged_tse <- Pre_processing_hashcodes(
    obj = tse_16SOTU,
    hashcodes = hashcodes_tse,
    merge_method = "max"
  )

  # Optional cleanup
  output_files <- c(
    file.path(tempdir(), "merged_physeq_sum_processed.rds"),
    file.path(tempdir(), "merged_tse_max_processed.rds")
  )
  for (f in output_files) {
    if (file.exists(f)) file.remove(f)
  }

  rm(subset_physeq, hashcodes_phy, merged_physeq,
     tse_16SOTU, subset_tse, hashcodes_tse, merged_tse, output_files, row_annots)
}
}
